==> zikomo-1.0.1-py3-none-any.whl: zikomo/helper.py <==
import os
import sys
sys.path.insert(0, os.path.dirname(os.getcwd()))

import json
import re
import argparse
from datetime import datetime
from zikomo.constants import *
from zikomo.utils import run, send_slack, send_email, get_random_image_url,SSH_HOSTS, SLACK_CHANNEL_ID_IMRAN,SLACK_CHANNEL_ID_GENERAL

# -------------------- CONFIG --------------------
image = ""
image_latest = ""
# ------------------------------------------------

def get_next_version(env):
    current_env=env if env.lower()!="prod" else "release"
    output = run("gh release list --limit 50")
    tags = [line.split()[0] for line in output.splitlines()]
    env_tags = [t for t in tags if re.match(rf"v\d+\.\d+\.\d+-{current_env}", t)]
    
    if not env_tags:
        print(f'Releases not found for {env}!')
        return f"v0.0.1-{env}"

    def numeric_version(tag):
        m = re.match(r"v(\d+)\.(\d+)\.(\d+)-", tag)
        if not m:
            return 0
        a, b, c = map(int, m.groups())
        return a * 100 + b * 10 + c

    latest_tag = max(env_tags, key=numeric_version)
    next_version_num = numeric_version(latest_tag) + 1

    a = next_version_num // 100
    b = (next_version_num // 10) % 10
    c = next_version_num % 10

    return f"v{a}.{b}.{c}-{current_env}"

def extract_pr_numbers_between_tags(from_tag, to_tag):
    print(f"🔍 Getting PRs from {from_tag} → {to_tag}")    
    run("git fetch --tags")
    log = run(f"git log --merges --pretty=format:'%s' {from_tag}..{to_tag}")
    return re.findall(r'#(\d+)', log)

def get_and_merge_prs(env):
    pr_nums, notes, remarks, slack_md = [], [], "", ""

    def format_pr_notes(num, title, body):
        pr_nums.append(f"#{num}")
        #notes.append(f"- PR #{num}: {title}\n{body}")
        slack_md_entry = f"*#{num}* – *{title}*\n"
        slack_remarks = ""
        
        for line in body.strip().splitlines():
            if line.strip():
                slack_remarks += f"• {line.strip()} "
                notes.append(f"• {line.strip()}")
                
        slack_remarks += "\n"
        return slack_md_entry, slack_remarks

    if env.lower() == "staging":
        raw = run("gh pr list --state open --limit 20 --json number,title,body,headRefName")
        prs = json.loads(raw)

        if not prs:
            return [], "", "", ""

        for pr in prs:
            num, title, body = pr["number"], pr["title"], pr.get("body", "")
            # Check mergeability
            pr_status = json.loads(run(f"gh pr view {num} --json mergeable"))
            if pr_status.get("mergeable") != "MERGEABLE":
                print(f"⚠️ Skipping PR #{num} – not mergeable.")
                continue

            run(f"gh pr merge {num} --merge")  # add --delete-branch if needed
            md, rmk = format_pr_notes(num, title, body)
            slack_md += md
            remarks += rmk

    else:  # UAT or LIVE
        tag_prefix = "release" if env.lower() == "prod" else env.lower()
            
        last_tag = run(f"git tag --sort=-creatordate | grep '{tag_prefix}$' | head -n1").strip()
        staging_tag = run(f"git tag --sort=-creatordate | grep 'staging$' | head -n1").strip()

        if not last_tag or not staging_tag:
            print("❌ Could not find suitable tags.")
            return [], "", "", ""

        pr_ids = extract_pr_numbers_between_tags(last_tag, staging_tag)
        if not pr_ids:
            return [], "", "", ""

        for pr_id in pr_ids:
            pr_data = json.loads(run(f"gh pr view {pr_id} --json number,title,body"))
            num, title, body = pr_data["number"], pr_data["title"], pr_data.get("body", "")
            md, rmk = format_pr_notes(num, title, body)
            slack_md += md
            remarks += rmk

    return pr_nums, "\n".join(notes), remarks.strip(), slack_md.strip()

def git_sync():
    run("git checkout master")
    run("git pull origin master")
    run("git push origin master")

def create_release(version):
    run(f"gh release create {version} --notes={version}")

def build_and_push_docker(image, image_latest, version):
    tag = version.replace("v", "")
    
    run(f"docker build -t {image_latest} -t {image} --build-arg VERSION={tag} .")
    run("az acr login --name zikomo")
    run(f"docker push {image}")
    run(f"docker push {image_latest}")

# DEPLOY
def deploy_to_server(env):
    host = SSH_HOSTS[env]
    cmd = f"sudo docker compose -f /var/www/mini/mini-stack.yml pull && sudo docker compose -f /var/www/mini/mini-stack.yml up -d"
    cmd_prune="sudo docker image prune -f"
    
    run(f'ssh {host} "{cmd}"')
    run(f'ssh {host} "{cmd_prune}"')

def log_release(version, image, pr_list, remarks):
    now = datetime.now()
    date = now.strftime("%d/%m/%Y")
    time = now.strftime("%I:%M %p")
    pr_str = ",".join(pr_list)
    remarks=remarks.replace("\r\n"," • ")
    
    row = f"| {date} | {time} | {image} | {version} | {pr_str} |{remarks} |\n"
    
    with open(RELEASE_DOCS_FILE, "a", encoding="utf-8") as f:
        f.write(row)
    
    print(f"📝 Appended release info to {RELEASE_DOCS_FILE}")

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--staging", action="store_true")
    parser.add_argument("--uat", action="store_true")
    parser.add_argument("--prod", action="store_true")
    return parser.parse_args()

def update_docs_site():
    os.chdir("E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs")
    
    run("git add .")
    run(f'git commit -m "info"')
    run("git push")
    run("mkdocs build")
    run("netlify deploy --prod --dir=site")

def set_docs_path(env,project):
    global RELEASE_DOCS_FILE    

    if env=="staging":
        RELEASE_DOCS_FILE=f"E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs\\docs\\api\\releases\\{project}\\staging-2025.md"
    elif env=="uat":
        RELEASE_DOCS_FILE=f"E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs\\docs\\api\\releases\\{project}\\uat-2025.md"
    else:
        RELEASE_DOCS_FILE=f"E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs\\docs\\api\\releases\\{project}\\production-2025.md"

def switch_dir(project):        
    match project.lower():
        case "flightbite":
          os.chdir(FLIGHTBITE_DIR)                          
          
        case "backoffice":
          os.chdir(BACKOFFICE_DIR)           
                       
        case "websites":
          os.chdir(WEBSITES_DIR) 
          
def set_version(env,project):
    global image,image_latest
    
    version=get_next_version(env)
    tag = version.replace("v", "")
    
    match project.lower():
        case "flightbite":                         
          image = f"{FLIGHTBITE_REGISTRY}:{tag}"
          image_latest = f"{FLIGHTBITE_REGISTRY}:{env}-latest"
          
        case "backoffice":          
          image = f"{BACKOFFICE_REGISTRY}:{tag}"
          image_latest = f"{BACKOFFICE_REGISTRY}:{env}-latest"
                       
        case "websites":          
          image = f"{WEBSITES_REGISTRY}:{tag}"
          image_latest = f"{WEBSITES_REGISTRY}:{env}-latest"                           
    
    return version

# MAIN WORKFLOW
def main_deploy(env,project):
    global image,image_latest
    
    switch_dir(project)            
    version = set_version(env,project)    
    print(f"\n🚀 Releasing to {env.upper()}: {version}")

    pr_nums, notes_text, remarks_text, slack_md = get_and_merge_prs(env)
    if not pr_nums:
        print("✅ All caught up! No PRs to deploy.")
        return
    
    git_sync()
    create_release(version)
    build_and_push_docker(image, image_latest, version)
    deploy_to_server(env)
    
    # LOG TO RELEASE NOTES
    set_docs_path(env,project)
    log_release(version, image, pr_nums, remarks_text)

    print("\n📚 Updating developer docs...")
    update_docs_site()
    
    # NOTIFY
    print("\n📢 Sending notifications...")
    image_url = get_random_image_url(env)
    send_slack(env,project, version, notes_text, image_url or "",SLACK_CHANNEL_ID_IMRAN)
    send_email(env, version, notes_text, project)

    print(f"\n✅ {version} deployed to {env}.")
    
if __name__ == "__main__":    
    #main_deploy("staging","websites")
    update_docs_site()
    #send_slack("Staging", "123", "1.Point1\r\n2.point2", IMAGE_HOST_URL+"staging/2.png")

==> zikomo-1.0.2-py3-none-any.whl: zikomo/helper.py <==
import os
import sys
sys.path.insert(0, os.path.dirname(os.getcwd()))

import json
import re
import argparse
from datetime import datetime
from zikomo.constants import *
from zikomo.utils import run, send_slack, send_email, get_random_image_url,SSH_HOSTS, SLACK_CHANNEL_ID_IMRAN,SLACK_CHANNEL_ID_GENERAL

# -------------------- CONFIG --------------------
image = ""
image_latest = ""
# ------------------------------------------------

def get_next_version(env):
    current_env=env if env.lower()!="prod" else "release"
    output = run("gh release list --limit 50")
    tags = [line.split()[0] for line in output.splitlines()]
    env_tags = [t for t in tags if re.match(rf"v\d+\.\d+\.\d+-{current_env}", t)]
    
    if not env_tags:
        print(f'Releases not found for {env}!')
        return f"v0.0.1-{env}"

    def numeric_version(tag):
        m = re.match(r"v(\d+)\.(\d+)\.(\d+)-", tag)
        if not m:
            return 0
        a, b, c = map(int, m.groups())
        return a * 100 + b * 10 + c

    latest_tag = max(env_tags, key=numeric_version)
    next_version_num = numeric_version(latest_tag) + 1

    a = next_version_num // 100
    b = (next_version_num // 10) % 10
    c = next_version_num % 10

    return f"v{a}.{b}.{c}-{current_env}"

def extract_pr_numbers_between_tags(from_tag, to_tag):
    print(f"🔍 Getting PRs from {from_tag} → {to_tag}")    
    run("git fetch --tags")
    log = run(f"git log --merges --pretty=format:'%s' {from_tag}..{to_tag}")
    return re.findall(r'#(\d+)', log)

def get_and_merge_prs(env):
    pr_nums, notes, remarks, slack_md = [], [], "", ""

    def format_pr_notes(num, title, body):
        pr_nums.append(f"#{num}")
        #notes.append(f"- PR #{num}: {title}\n{body}")
        slack_md_entry = f"*#{num}* – *{title}*\n"
        slack_remarks = ""
        
        for line in body.strip().splitlines():
            if line.strip():
                slack_remarks += f"• {line.strip()} "
                notes.append(f"• {line.strip()}")
                
        slack_remarks += "\n"
        return slack_md_entry, slack_remarks

    if env.lower() == "staging":
        raw = run("gh pr list --state open --limit 20 --json number,title,body,headRefName")
        prs = json.loads(raw)

        if not prs:
            return [], "", "", ""

        for pr in prs:
            num, title, body = pr["number"], pr["title"], pr.get("body", "")
            # Check mergeability
            pr_status = json.loads(run(f"gh pr view {num} --json mergeable"))
            if pr_status.get("mergeable") != "MERGEABLE":
                print(f"⚠️ Skipping PR #{num} – not mergeable.")
                continue

            run(f"gh pr merge {num} --merge")  # add --delete-branch if needed
            md, rmk = format_pr_notes(num, title, body)
            slack_md += md
            remarks += rmk

    else:  # UAT or LIVE
        tag_prefix = "release" if env.lower() == "prod" else env.lower()
            
        last_tag = run(f"git tag --sort=-creatordate | grep '{tag_prefix}$' | head -n1").strip()
        staging_tag = run(f"git tag --sort=-creatordate | grep 'staging$' | head -n1").strip()

        if not last_tag or not staging_tag:
            print("❌ Could not find suitable tags.")
            return [], "", "", ""

        pr_ids = extract_pr_numbers_between_tags(last_tag, staging_tag)
        if not pr_ids:
            return [], "", "", ""

        for pr_id in pr_ids:
            pr_data = json.loads(run(f"gh pr view {pr_id} --json number,title,body"))
            num, title, body = pr_data["number"], pr_data["title"], pr_data.get("body", "")
            md, rmk = format_pr_notes(num, title, body)
            slack_md += md
            remarks += rmk

    return pr_nums, "\n".join(notes), remarks.strip(), slack_md.strip()

def git_sync():
    run("git checkout master")
    run("git pull origin master")
    run("git push origin master")

def create_release(version):
    run(f"gh release create {version} --notes={version}")

def build_and_push_docker(image, image_latest, version):
    tag = version.replace("v", "")
    
    run(f"docker build -t {image_latest} -t {image} --build-arg VERSION={tag} .")
    run("az acr login --name zikomo")
    run(f"docker push {image}")
    run(f"docker push {image_latest}")

# DEPLOY
def deploy_to_server(env):
    host = SSH_HOSTS[env]
    cmd = f"sudo docker compose -f /var/www/mini/mini-stack.yml pull && sudo docker compose -f /var/www/mini/mini-stack.yml up -d"
    cmd_prune="sudo docker image prune -f"
    
    run(f'ssh {host} "{cmd}"')
    run(f'ssh {host} "{cmd_prune}"')

def log_release(version, image, pr_list, remarks):
    now = datetime.now()
    date = now.strftime("%d/%m/%Y")
    time = now.strftime("%I:%M %p")
    pr_str = ",".join(pr_list)
    remarks=remarks.replace("\r\n"," • ")
    
    row = f"| {date} | {time} | {image} | {version} | {pr_str} |{remarks} |\n"
    
    with open(RELEASE_DOCS_FILE, "a", encoding="utf-8") as f:
        f.write(row)
    
    print(f"📝 Appended release info to {RELEASE_DOCS_FILE}")

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--staging", action="store_true")
    parser.add_argument("--uat", action="store_true")
    parser.add_argument("--prod", action="store_true")
    return parser.parse_args()

def update_docs_site():
    os.chdir("E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs")
    
    run("git add .")
    run(f'git commit -m "info"')
    run("git push")
    run("mkdocs build")
    run("netlify deploy --prod --dir=site")

def set_docs_path(env,project):
    global RELEASE_DOCS_FILE    

    if env=="staging":
        RELEASE_DOCS_FILE=f"E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs\\docs\\api\\releases\\{project}\\staging-2025.md"
    elif env=="uat":
        RELEASE_DOCS_FILE=f"E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs\\docs\\api\\releases\\{project}\\uat-2025.md"
    else:
        RELEASE_DOCS_FILE=f"E:\\ZikomoSolutions\\Projects\\ZikomoDocs\\developer-docs\\docs\\api\\releases\\{project}\\production-2025.md"

def switch_dir(project):        
    match project.lower():
        case "flightbite":
          os.chdir(FLIGHTBITE_DIR)                          
          
        case "backoffice":
          os.chdir(BACKOFFICE_DIR)           
                       
        case "websites":
          os.chdir(WEBSITES_DIR) 
          
def set_version(env,project):
    global image,image_latest
    
    version=get_next_version(env)
    tag = version.replace("v", "")
    
    match project.lower():
        case "flightbite":                         
          image = f"{FLIGHTBITE_REGISTRY}:{tag}"
          image_latest = f"{FLIGHTBITE_REGISTRY}:{env}-latest"
          
        case "backoffice":          
          image = f"{BACKOFFICE_REGISTRY}:{tag}"
          image_latest = f"{BACKOFFICE_REGISTRY}:{env}-latest"
                       
        case "websites":          
          image = f"{WEBSITES_REGISTRY}:{tag}"
          image_latest = f"{WEBSITES_REGISTRY}:{env}-latest"                           
    
    return version

# MAIN WORKFLOW
def main_deploy(env,project):
    global image,image_latest
    
    switch_dir(project)            
    version = set_version(env,project)    
    print(f"\n🚀 Releasing to {env.upper()}: {version}")

    pr_nums, notes_text, remarks_text, slack_md = get_and_merge_prs(env)
    if not pr_nums:
        print("✅ All caught up! No PRs to deploy.")
        return
    
    git_sync()
    create_release(version)
    build_and_push_docker(image, image_latest, version)
    deploy_to_server(env)
    
    # LOG TO RELEASE NOTES
    set_docs_path(env,project)
    log_release(version, image, pr_nums, remarks_text)

    print("\n📚 Updating developer docs...")
    update_docs_site()
    
    # NOTIFY
    print("\n📢 Sending notifications...")
    image_url = get_random_image_url(env.lower())
    send_slack(env,project, version, notes_text, image_url or "",SLACK_CHANNEL_ID_IMRAN)
    send_email(env, version, notes_text, project)

    print(f"\n✅ {version} deployed to {env}.")
    
if __name__ == "__main__":    
    #main_deploy("staging","websites")
    update_docs_site()
    #send_slack("Staging", "123", "1.Point1\r\n2.point2", IMAGE_HOST_URL+"staging/2.png")
